==> tombi-0.1.35.tar.gz: tombi-0.1.35/crates/config/src/schema.rs <==
use toml_version::TomlVersion;

use crate::{Enabled, OneOrMany, SchemaCatalogPath};

#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
#[cfg_attr(feature = "serde", serde(deny_unknown_fields))]
#[cfg_attr(feature = "serde", serde(rename_all = "kebab-case"))]
#[cfg_attr(feature = "jsonschema", derive(schemars::JsonSchema))]
#[derive(Debug, Default, Clone)]
pub struct SchemaOptions {
    /// # Enable or disable the schema.
    #[cfg_attr(feature = "jsonschema", schemars(default = "Enabled::default"))]
    pub enabled: Option<Enabled>,

    /// # Schema catalog options.
    pub catalog: Option<SchemaCatalog>,
}

impl SchemaOptions {
    pub const fn default() -> Self {
        Self {
            enabled: None,
            catalog: None,
        }
    }

    pub fn catalog_paths(&self) -> Option<Vec<SchemaCatalogPath>> {
        if self.enabled.unwrap_or_default().value() {
            Some(
                self.catalog
                    .as_ref()
                    .and_then(|catalog| catalog.path.as_ref().map(|path| path.as_ref().to_vec()))
                    .unwrap_or_else(|| vec![SchemaCatalogPath::default()]),
            )
        } else {
            None
        }
    }
}

#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
#[cfg_attr(feature = "serde", serde(deny_unknown_fields))]
#[cfg_attr(feature = "serde", serde(rename_all = "kebab-case"))]
#[cfg_attr(feature = "jsonschema", derive(schemars::JsonSchema))]
#[derive(Debug, Default, Clone)]
pub struct SchemaCatalog {
    /// # The schema catalog path or url.
    ///
    /// You can specify multiple catalogs by making it an array.
    /// If you want to disable the default catalog, specify an empty array.
    #[cfg_attr(
        feature = "jsonschema",
        schemars(default = "SchemaCatalogPath::default")
    )]
    pub path: Option<OneOrMany<SchemaCatalogPath>>,
}

#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
#[cfg_attr(feature = "serde", serde(deny_unknown_fields))]
#[cfg_attr(feature = "serde", serde(rename_all = "kebab-case"))]
#[cfg_attr(feature = "jsonschema", derive(schemars::JsonSchema))]
#[derive(Debug, Clone)]
pub struct SchemaCatalogItem {
    /// # The TOML version that the schema is available.
    pub toml_version: Option<TomlVersion>,

    /// # The schema path.
    pub path: String,

    /// # The file match pattern of the schema.
    ///
    /// The file match pattern to include the target to apply the schema.
    /// Supports glob pattern.
    #[cfg_attr(feature = "jsonschema", schemars(length(min = 1)))]
    pub include: Option<Vec<String>>,
    // /// # The schema options for specific keys.
    // #[cfg_attr(feature = "jsonschema", schemars(default))]
    // subschemas: Option<Vec<SubSchemaOptions>>,
}

// #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
// #[cfg_attr(feature = "serde", serde(deny_unknown_fields))]
// #[cfg_attr(feature = "serde", serde(rename_all = "kebab-case"))]
// #[cfg_attr(feature = "jsonschema", derive(schemars::JsonSchema))]
// #[derive(Debug, Clone)]
// pub struct SubSchemaOptions {
//     /// # The schema path.
//     path: String,

//     /// The path of the key to apply the schema.
//     keys: String,
// }

#[cfg(test)]
mod tests {
    use assert_matches::assert_matches;

    use super::*;

    #[test]
    fn schema_catalog_paths_default() {
        let schema = SchemaOptions::default();
        let _expected = [SchemaCatalogPath::default()];

        assert_matches!(schema.catalog_paths(), Some(_expected));
    }

    #[test]
    fn schema_catalog_paths_empty() {
        let schema = SchemaOptions {
            catalog: Some(SchemaCatalog {
                path: Some(vec![].into()),
            }),
            ..Default::default()
        };

        let _expected: Vec<SchemaCatalogPath> = vec![];

        assert_matches!(schema.catalog_paths(), Some(_expected));
    }
}

==> tombi-0.2.20.tar.gz: tombi-0.2.20/crates/config/src/schema.rs <==
use toml_version::TomlVersion;

use crate::{Enabled, OneOrMany, SchemaCatalogPath};

#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
#[cfg_attr(feature = "serde", serde(deny_unknown_fields))]
#[cfg_attr(feature = "serde", serde(rename_all = "kebab-case"))]
#[cfg_attr(feature = "jsonschema", derive(schemars::JsonSchema))]
#[cfg_attr(feature = "jsonschema", schemars(extend("x-tombi-table-keys-order" = x_tombi::TableKeysOrder::Schema)))]
#[derive(Debug, Default, Clone)]
pub struct SchemaOptions {
    /// # Enable or disable the schema.
    #[cfg_attr(feature = "jsonschema", schemars(default = "Enabled::default"))]
    pub enabled: Option<Enabled>,

    /// # Schema catalog options.
    pub catalog: Option<SchemaCatalog>,
}

impl SchemaOptions {
    pub const fn default() -> Self {
        Self {
            enabled: None,
            catalog: None,
        }
    }

    pub fn catalog_paths(&self) -> Option<Vec<SchemaCatalogPath>> {
        if self.enabled.unwrap_or_default().value() {
            Some(
                self.catalog
                    .as_ref()
                    .and_then(|catalog| catalog.path.as_ref().map(|path| path.as_ref().to_vec()))
                    .unwrap_or_else(|| vec![SchemaCatalogPath::default()]),
            )
        } else {
            None
        }
    }
}

#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
#[cfg_attr(feature = "serde", serde(deny_unknown_fields))]
#[cfg_attr(feature = "serde", serde(rename_all = "kebab-case"))]
#[cfg_attr(feature = "jsonschema", derive(schemars::JsonSchema))]
#[derive(Debug, Default, Clone)]
pub struct SchemaCatalog {
    /// # The schema catalog path or url.
    ///
    /// The catalog is evaluated after the schemas specified by [[schemas]].
    ///
    /// You can specify multiple catalogs by making it an array.
    /// If you specify an array, the catalogs are searched in order of priority starting from the first catalog.
    /// If you want to disable the default catalog, specify an empty array.
    #[cfg_attr(
        feature = "jsonschema",
        schemars(default = "SchemaCatalogPath::default")
    )]
    pub path: Option<OneOrMany<SchemaCatalogPath>>,
}

#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
#[cfg_attr(feature = "serde", serde(untagged))]
#[cfg_attr(feature = "jsonschema", derive(schemars::JsonSchema))]
#[derive(Debug, Clone)]
pub enum Schema {
    Root(RootSchema),
    Sub(SubSchema),
}

impl Schema {
    pub fn path(&self) -> &str {
        match self {
            Self::Root(item) => &item.path,
            Self::Sub(item) => &item.path,
        }
    }

    pub fn include(&self) -> &[String] {
        match self {
            Self::Root(item) => &item.include,
            Self::Sub(item) => &item.include,
        }
    }

    pub fn toml_version(&self) -> Option<TomlVersion> {
        match self {
            Self::Root(item) => item.toml_version,
            Self::Sub(_) => None,
        }
    }

    pub fn root_keys(&self) -> Option<&str> {
        match self {
            Self::Root(_) => None,
            Self::Sub(item) => item.root_keys.as_deref(),
        }
    }
}

#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
#[cfg_attr(feature = "serde", serde(deny_unknown_fields))]
#[cfg_attr(feature = "serde", serde(rename_all = "kebab-case"))]
#[cfg_attr(feature = "jsonschema", derive(schemars::JsonSchema))]
#[derive(Debug, Clone)]
pub struct RootSchema {
    /// # The TOML version that the schema is available.
    pub toml_version: Option<TomlVersion>,

    /// # The schema path.
    pub path: String,

    /// # The file match pattern of the schema.
    ///
    /// The file match pattern to include the target to apply the schema.
    /// Supports glob pattern.
    #[cfg_attr(feature = "jsonschema", schemars(length(min = 1)))]
    pub include: Vec<String>,
}

#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
#[cfg_attr(feature = "serde", serde(deny_unknown_fields))]
#[cfg_attr(feature = "serde", serde(rename_all = "kebab-case"))]
#[cfg_attr(feature = "jsonschema", derive(schemars::JsonSchema))]
#[derive(Debug, Clone)]
pub struct SubSchema {
    /// # The schema path.
    pub path: String,

    /// # The file match pattern of the schema.
    ///
    /// The file match pattern to include the target to apply the schema.
    /// Supports glob pattern.
    #[cfg_attr(feature = "jsonschema", schemars(length(min = 1)))]
    pub include: Vec<String>,

    /// # The keys to apply the schema.
    #[cfg_attr(feature = "jsonschema", schemars(length(min = 1)))]
    pub root_keys: Option<String>,
}

#[cfg(test)]
mod tests {
    use assert_matches::assert_matches;

    use super::*;

    #[test]
    fn schema_catalog_paths_default() {
        let schema = SchemaOptions::default();
        let _expected = [SchemaCatalogPath::default()];

        assert_matches!(schema.catalog_paths(), Some(_expected));
    }

    #[test]
    fn schema_catalog_paths_empty() {
        let schema = SchemaOptions {
            catalog: Some(SchemaCatalog {
                path: Some(vec![].into()),
            }),
            ..Default::default()
        };

        let _expected: Vec<SchemaCatalogPath> = vec![];

        assert_matches!(schema.catalog_paths(), Some(_expected));
    }
}
